/*
 * Copyright (c) 2000-2002, 2004 Silicon Graphics, Inc All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it would be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place - Suite 330, Boston MA 02111-1307, USA.
 *
 * Contact information:  Silicon Graphics, Inc., 1500 Crittenden Lane,
 * Mountain View, CA  94043, or:
 *
 * http://www.sgi.com
 */

#ifndef _JOBCTL_H
#define _JOBCTL_H

#include <stdint.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <errno.h>

/* These typedefs and defines are necessary for the paggctl include below */
typedef u_int64_t u64;
typedef u_int32_t u32;
struct task_struct;

#include <linux/jobctl.h>

/* The jobctl() function communicates with the kernel by a
 * set of ioctl calls via the /proc/job file.
 */
static int jobctl(int request, void *data) {
    static int job_ioctl_fd = -2;

    int rc;

    if (job_ioctl_fd == -2) {
	char fname[256];
	sprintf(fname, "/proc/%s", JOB_PROC_ENTRY);
	job_ioctl_fd = open(fname,0);
	if (job_ioctl_fd < 0) {
		fprintf(stderr, "Can't open proc file %s\n", fname);
		return -1;
	}
    }

    rc = ioctl(job_ioctl_fd, request, data);
    if (rc < 0) {
	    return -errno;
    }

    return 0;
}

#endif /* _JOBCTL_H */
